import torch.nn as nn
import math
import torch.utils.model_zoo as model_zoo
import mmd
import torch.nn.functional as F
from torch.autograd import Variable
import torch
import numpy as np

__all__ = ['ResNet', 'resnet50']

model_urls = {
    'resnet50': 'https://download.pytorch.org/models/resnet50-19c8e357.pth',
}

class GradientReverseLayer(torch.autograd.Function):
    def __init__(self, iter_num=0, alpha=1.0, low_value=0.0, high_value=0.1, max_iter=1000.0):
        self.iter_num = iter_num
        self.alpha = alpha
        self.low_value = low_value
        self.high_value = high_value
        self.max_iter = max_iter

    def forward(self, input):
        self.iter_num += 1
        output = input * 1.0
        return output

    def backward(self, grad_output):
        self.coeff = np.float(
            2.0 * (self.high_value - self.low_value) / (1.0 + np.exp(-self.alpha * self.iter_num / self.max_iter)) - (
                        self.high_value - self.low_value) + self.low_value)
        return -self.coeff * grad_output
    
def conv3x3(in_planes, out_planes, stride=1):
    """3x3 convolution with padding"""
    return nn.Conv2d(in_planes, out_planes, kernel_size=3, stride=stride,
                     padding=1, bias=False)

class BasicBlock(nn.Module):
    expansion = 1

    def __init__(self, inplanes, planes, stride=1, downsample=None):
        super(BasicBlock, self).__init__()
        self.conv1 = conv3x3(inplanes, planes, stride)
        self.bn1 = nn.BatchNorm2d(planes)
        self.relu = nn.ReLU(inplace=True)
        self.conv2 = conv3x3(planes, planes)
        self.bn2 = nn.BatchNorm2d(planes)
        self.downsample = downsample
        self.stride = stride

    def forward(self, x):
        residual = x

        out = self.conv1(x)
        out = self.bn1(out)
        out = self.relu(out)

        out = self.conv2(out)
        out = self.bn2(out)

        if self.downsample is not None:
            residual = self.downsample(x)

        out += residual
        out = self.relu(out)

        return out


class Bottleneck(nn.Module):
    expansion = 4

    def __init__(self, inplanes, planes, stride=1, downsample=None):
        super(Bottleneck, self).__init__()
        self.conv1 = nn.Conv2d(inplanes, planes, kernel_size=1, bias=False)
        self.bn1 = nn.BatchNorm2d(planes)
        self.conv2 = nn.Conv2d(planes, planes, kernel_size=3, stride=stride,
                               padding=1, bias=False)
        self.bn2 = nn.BatchNorm2d(planes)
        self.conv3 = nn.Conv2d(planes, planes * 4, kernel_size=1, bias=False)
        self.bn3 = nn.BatchNorm2d(planes * 4)
        self.relu = nn.ReLU(inplace=True)
        self.downsample = downsample
        self.stride = stride

    def forward(self, x):
        residual = x

        out = self.conv1(x)
        out = self.bn1(out)
        out = self.relu(out)

        out = self.conv2(out)
        out = self.bn2(out)
        out = self.relu(out)

        out = self.conv3(out)
        out = self.bn3(out)

        if self.downsample is not None:
            residual = self.downsample(x)

        out += residual
        out = self.relu(out)

        return out

class ADDneck(nn.Module):

    def __init__(self, inplanes, planes, stride=1, downsample=None):
        super(ADDneck, self).__init__()
        self.conv1 = nn.Conv2d(inplanes, planes, kernel_size=1, bias=False)
        self.bn1 = nn.BatchNorm2d(planes)
        self.conv2 = nn.Conv2d(planes, planes, kernel_size=3, stride=stride,padding=1, bias=False)
        self.bn2 = nn.BatchNorm2d(planes)
        self.conv3 = nn.Conv2d(planes, planes, kernel_size=1, bias=False)
        self.bn3 = nn.BatchNorm2d(planes)
        self.relu = nn.ReLU(inplace=True)
        self.stride = stride

    def forward(self, x):

        out = self.conv1(x)
        out = self.bn1(out)
        out = self.relu(out)

        out = self.conv2(out)
        out = self.bn2(out)
        out = self.relu(out)

        out = self.conv3(out)
        out = self.bn3(out)
        out = self.relu(out)

        return out

class ResNet(nn.Module):

    def __init__(self, block, layers, num_classes=1000):
        self.inplanes = 64
        super(ResNet, self).__init__()
        self.conv1 = nn.Conv2d(3, 64, kernel_size=7, stride=2, padding=3,bias=False)
        self.bn1 = nn.BatchNorm2d(64)
        self.relu = nn.ReLU(inplace=True)
        self.maxpool = nn.MaxPool2d(kernel_size=3, stride=2, padding=1)
        self.layer1 = self._make_layer(block, 64, layers[0])
        self.layer2 = self._make_layer(block, 128, layers[1], stride=2)
        self.layer3 = self._make_layer(block, 256, layers[2], stride=2)
        self.layer4 = self._make_layer(block, 512, layers[3], stride=2)
        self.avgpool = nn.AvgPool2d(7, stride=1)
        self.baselayer = [self.conv1, self.bn1, self.layer1, self.layer2, self.layer3, self.layer4]
        self.fc = nn.Linear(512 * block.expansion, num_classes)

        for m in self.modules():
            if isinstance(m, nn.Conv2d):
                n = m.kernel_size[0] * m.kernel_size[1] * m.out_channels
                m.weight.data.normal_(0, math.sqrt(2. / n))
            elif isinstance(m, nn.BatchNorm2d):
                m.weight.data.fill_(1)
                m.bias.data.zero_()

    def _make_layer(self, block, planes, blocks, stride=1):
        downsample = None
        if stride != 1 or self.inplanes != planes * block.expansion:
            downsample = nn.Sequential(
                nn.Conv2d(self.inplanes, planes * block.expansion,kernel_size=1, stride=stride, bias=False),
                nn.BatchNorm2d(planes * block.expansion),
            )

        layers = []
        layers.append(block(self.inplanes, planes, stride, downsample))
        self.inplanes = planes * block.expansion
        for i in range(1, blocks):
            layers.append(block(self.inplanes, planes))

        return nn.Sequential(*layers)

    def forward(self, x):
        x = self.conv1(x)
        x = self.bn1(x)
        x = self.relu(x)
        x = self.maxpool(x)

        x = self.layer1(x)
        x = self.layer2(x)
        x = self.layer3(x)
        x = self.layer4(x)

        return x
class MYDNN(nn.Module):

    def __init__(self, num_classes=31):
        super(MYDNN, self).__init__()
        self.sharedNet = resnet50(True)
        self.sonnet1 = ADDneck(2048, 256)
        self.sonnet2 = ADDneck(2048, 256)
        #self.cls_fc_son1 = nn.Linear(256, num_classes)
        #self.cls_fc_son2 = nn.Linear(256, num_classes)
        self.avgpool = nn.AvgPool2d(7, stride=1)
        self.grl_layer = GradientReverseLayer()
        '''
        ###  分类器的层数到底是两层还是一层，一会可以做实验看看 #######
        self.classifier_layer_list_11 = [nn.Linear(256, 256), nn.ReLU(), nn.Dropout(0.5),nn.Linear(256, num_classes)]
        self.classifier_layer_son1_1 = nn.Sequential(*self.classifier_layer_list_11)
        self.classifier_layer_list_12 = [nn.Linear(256, 256), nn.ReLU(), nn.Dropout(0.5),nn.Linear(256, num_classes)]
        self.classifier_layer_son1_2 = nn.Sequential(*self.classifier_layer_list_12)
        
        self.classifier_layer_list_21 = [nn.Linear(256, 256), nn.ReLU(), nn.Dropout(0.5),nn.Linear(256, num_classes)]
        self.classifier_layer_son2_1 = nn.Sequential(*self.classifier_layer_list_21)
        self.classifier_layer_list_22 = [nn.Linear(256, 256), nn.ReLU(), nn.Dropout(0.5),nn.Linear(256, num_classes)]
        self.classifier_layer_son2_2 = nn.Sequential(*self.classifier_layer_list_22)
        '''
        self.classifier_layer_list_11 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son1_1 = nn.Sequential(*self.classifier_layer_list_11)
        self.classifier_layer_list_12 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son1_2 = nn.Sequential(*self.classifier_layer_list_12)
        
        self.classifier_layer_list_21 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son2_1 = nn.Sequential(*self.classifier_layer_list_21)
        self.classifier_layer_list_22 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son2_2 = nn.Sequential(*self.classifier_layer_list_22)
        
        self.softmax = nn.Softmax(dim=1)

        ## initialization
        '''
        for dep in range(2):
            self.classifier_layer_son1_1[dep * 3].weight.data.normal_(0, 0.01)
            self.classifier_layer_son1_1[dep * 3].bias.data.fill_(0.0)
            
            self.classifier_layer_son1_2[dep * 3].weight.data.normal_(0, 0.01)
            self.classifier_layer_son1_2[dep * 3].bias.data.fill_(0.0)
            
            self.classifier_layer_son2_1[dep * 3].weight.data.normal_(0, 0.01)
            self.classifier_layer_son2_1[dep * 3].bias.data.fill_(0.0)
            
            self.classifier_layer_son2_2[dep * 3].weight.data.normal_(0, 0.01)
            self.classifier_layer_son2_2[dep * 3].bias.data.fill_(0.0)
        '''

    def forward(self, data_src, data_tgt = 0, label_src = 0, mark = 1):
        mmd_loss = 0
        if self.training == True:
            if mark == 1:
                data_src = self.sharedNet(data_src)
                data_tgt = self.sharedNet(data_tgt)

                data_tgt_son1 = self.sonnet1(data_tgt)
                data_tgt_son1 = self.avgpool(data_tgt_son1)
                data_tgt_son1 = data_tgt_son1.view(data_tgt_son1.size(0), -1)

                data_src = self.sonnet1(data_src)
                data_src = self.avgpool(data_src)
                data_src = data_src.view(data_src.size(0), -1)
                
                inputs = torch.cat((data_src, data_tgt_son1), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son1_2(features_adv)
        
                outputs = self.classifier_layer_son1_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src.size(0)), label_src)## 源域分类损失
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt

                return classifier_loss, transfer_loss

            if mark == 2:
                data_src = self.sharedNet(data_src)
                data_tgt = self.sharedNet(data_tgt)

                data_tgt_son2 = self.sonnet2(data_tgt)
                data_tgt_son2 = self.avgpool(data_tgt_son2)
                data_tgt_son2 = data_tgt_son2.view(data_tgt_son2.size(0), -1)

                data_src = self.sonnet2(data_src)
                data_src = self.avgpool(data_src)
                data_src = data_src.view(data_src.size(0), -1)
                
                inputs = torch.cat((data_src, data_tgt_son2), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son2_2(features_adv)
        
                outputs = self.classifier_layer_son2_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src.size(0)), label_src)
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt

                return classifier_loss, transfer_loss

        else:
            data = self.sharedNet(data_src)

            fea_son1 = self.sonnet1(data)
            fea_son1 = self.avgpool(fea_son1)
            fea_son1 = fea_son1.view(fea_son1.size(0), -1)
            pred1 = self.classifier_layer_son1_1(fea_son1)

            fea_son2 = self.sonnet2(data)
            fea_son2 = self.avgpool(fea_son2)
            fea_son2 = fea_son2.view(fea_son2.size(0), -1)
            pred2 = self.classifier_layer_son2_1(fea_son2)

            return pred1, pred2

class MYDNN_MOE(nn.Module):
    def __init__(self, num_classes=31):
        super(MYDNN_MOE, self).__init__()
        #self.sharedNet = resnet50(True)
        self.sharedNet = alexnet(True)
        self.sonnet1 = ADDneck(2048, 256)
        self.sonnet2 = ADDneck(2048, 256)
        self.avgpool = nn.AvgPool2d(7, stride=1)
        self.grl_layer = GradientReverseLayer()
       
        self.classifier_layer_list_11 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son1_1 = nn.Sequential(*self.classifier_layer_list_11)
        self.classifier_layer_list_12 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son1_2 = nn.Sequential(*self.classifier_layer_list_12)
        
        self.classifier_layer_list_21 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son2_1 = nn.Sequential(*self.classifier_layer_list_21)
        self.classifier_layer_list_22 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son2_2 = nn.Sequential(*self.classifier_layer_list_22)
        
        self.softmax = nn.Softmax(dim=1)

    def forward(self, data_src1,data_src2=0,data_tgt = 0,label_src1 = 0,label_src2=0,mark = 1):
        mmd_loss = 0
        if self.training == True:
            if mark == 1:
                data_src1 = self.sharedNet(data_src1)
                data_tgt = self.sharedNet(data_tgt)
                data_src2 = self.sharedNet(data_src2)

                data_tgt_son1 = self.sonnet1(data_tgt)
                data_tgt_son1 = self.avgpool(data_tgt_son1)
                data_tgt_son1 = data_tgt_son1.view(data_tgt_son1.size(0), -1)

                data_src1 = self.sonnet1(data_src1)
                data_src1 = self.avgpool(data_src1)
                data_src1 = data_src1.view(data_src1.size(0), -1)
                ###############  为了moe而加的   ############ 
                data_src2 = self.sonnet2(data_src2)
                data_src2 = self.avgpool(data_src2)
                data_src2 = data_src2.view(data_src2.size(0), -1)
                
                
                inputs = torch.cat((data_src1, data_tgt_son1), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son1_2(features_adv)
        
                outputs = self.classifier_layer_son1_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src1.size(0)), label_src1)## 源域分类损失
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src1.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src1.size(0), inputs.size(0) - label_src1.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src1.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src1.size(0), inputs.size(0) - label_src1.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt
                
                outpus_data_src1 = self.classifier_layer_son2_1(data_src1)
                return classifier_loss, transfer_loss, data_src1,outpus_data_src1

            if mark == 2:
                data_src2 = self.sharedNet(data_src2)
                data_tgt = self.sharedNet(data_tgt)

                data_tgt_son2 = self.sonnet2(data_tgt)
                data_tgt_son2 = self.avgpool(data_tgt_son2)
                data_tgt_son2 = data_tgt_son2.view(data_tgt_son2.size(0), -1)

                data_src2 = self.sonnet2(data_src2)
                data_src2 = self.avgpool(data_src2)
                data_src2 = data_src2.view(data_src2.size(0), -1)
                
                inputs = torch.cat((data_src2, data_tgt_son2), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son2_2(features_adv)
        
                outputs = self.classifier_layer_son2_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src2.size(0)), label_src2)
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src2.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src2.size(0), inputs.size(0) - label_src2.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src2.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src2.size(0), inputs.size(0) - label_src2.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt
                
                outpus_data_src2 = self.classifier_layer_son1_1(data_src2)
                return classifier_loss, transfer_loss, data_src2,outpus_data_src2

        else:             
            data = self.sharedNet(data_src1)

            fea_son1 = self.sonnet1(data)
            fea_son1 = self.avgpool(fea_son1)
            fea_son1 = fea_son1.view(fea_son1.size(0), -1)
            pred1 = self.classifier_layer_son1_1(fea_son1)

            fea_son2 = self.sonnet2(data)
            fea_son2 = self.avgpool(fea_son2)
            fea_son2 = fea_son2.view(fea_son2.size(0), -1)
            pred2 = self.classifier_layer_son2_1(fea_son2)
            return pred1, pred2,fea_son1,fea_son2

class MYDNN_COM_MOE(nn.Module):
    def __init__(self, num_classes=31):
        super(MYDNN_COM_MOE, self).__init__()
        self.sharedNet = resnet50(True)
        #self.bottleneck = nn.Linear(self.sharedNet.fc.in_features, 256)
        
        self.bottleneck = ADDneck(2048, 256)
        #self.sonnet1 = ADDneck(2048, 256)
        #self.sonnet2 = ADDneck(2048, 256)
        self.avgpool = nn.AvgPool2d(7, stride=1)
        self.grl_layer = GradientReverseLayer()
       
        self.classifier_layer_list_11 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son1_1 = nn.Sequential(*self.classifier_layer_list_11)
        self.classifier_layer_list_12 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son1_2 = nn.Sequential(*self.classifier_layer_list_12)
        
        self.classifier_layer_list_21 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son2_1 = nn.Sequential(*self.classifier_layer_list_21)
        self.classifier_layer_list_22 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son2_2 = nn.Sequential(*self.classifier_layer_list_22)
        
        self.softmax = nn.Softmax(dim=1)

    def forward(self, data_src1,data_src2=0,data_tgt = 0,label_src1 = 0,label_src2=0,mark = 1):
        mmd_loss = 0
        if self.training == True:
            if mark == 1:
                data_src1 = self.sharedNet(data_src1)
                data_tgt = self.sharedNet(data_tgt)
                data_src2 = self.sharedNet(data_src2)


                #data_tgt_son1 = self.sonnet1(data_tgt)
                #data_tgt_son1 = self.avgpool(data_tgt_son1)
                data_tgt_son1 = self.bottleneck(data_tgt)
                data_tgt_son1 = self.avgpool(data_tgt_son1)
                data_tgt_son1 = data_tgt_son1.view(data_tgt_son1.size(0), -1)
				
                data_src1 = self.bottleneck(data_src1)
                data_src1 = self.avgpool(data_src1)
                data_src1 = data_src1.view(data_src1.size(0), -1)
                ###############  为了moe而加的   ############ 
                data_src2 = self.bottleneck(data_src2)
                data_src2 = self.avgpool(data_src2)
                data_src2 = data_src2.view(data_src2.size(0), -1)
                
                
                inputs = torch.cat((data_src1, data_tgt_son1), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son1_2(features_adv)
        
                outputs = self.classifier_layer_son1_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src1.size(0)), label_src1)## 源域分类损失
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src1.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src1.size(0), inputs.size(0) - label_src1.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src1.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src1.size(0), inputs.size(0) - label_src1.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt
                
                outpus_data_src1 = self.classifier_layer_son2_1(data_src1)
                return classifier_loss, transfer_loss, data_src1,outpus_data_src1

            if mark == 2:
                data_src2 = self.sharedNet(data_src2)
                data_tgt = self.sharedNet(data_tgt)

                data_tgt_son2 = self.bottleneck(data_tgt)
                data_tgt_son2 = self.avgpool(data_tgt_son2)
                data_tgt_son2 = data_tgt_son2.view(data_tgt_son2.size(0), -1)

                data_src2 = self.bottleneck(data_src2)
                data_src2 = self.avgpool(data_src2)
                data_src2 = data_src2.view(data_src2.size(0), -1)
                
                inputs = torch.cat((data_src2, data_tgt_son2), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son2_2(features_adv)
        
                outputs = self.classifier_layer_son2_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src2.size(0)), label_src2)
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src2.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src2.size(0), inputs.size(0) - label_src2.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src2.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src2.size(0), inputs.size(0) - label_src2.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt
                
                outpus_data_src2 = self.classifier_layer_son1_1(data_src2)
                return classifier_loss, transfer_loss, data_src2,outpus_data_src2

        else:             
            data = self.sharedNet(data_src1)

            fea_son1 = self.bottleneck(data)
            fea_son1 = self.avgpool(fea_son1)
            fea_son1 = fea_son1.view(fea_son1.size(0), -1)
            pred1 = self.classifier_layer_son1_1(fea_son1)

            fea_son2 = self.bottleneck(data)
            fea_son2 = self.avgpool(fea_son2)
            fea_son2 = fea_son2.view(fea_son2.size(0), -1)
            pred2 = self.classifier_layer_son2_1(fea_son2)
            return pred1, pred2,fea_son1,fea_son2


class MYDNN_Conditional(nn.Module):

    def __init__(self, num_classes=31):
        super(MYDNN_Conditional, self).__init__()
        self.sharedNet = resnet50(True)
        self.sonnet1 = ADDneck(2048, 256)
        self.sonnet2 = ADDneck(2048, 256)
        self.avgpool = nn.AvgPool2d(7, stride=1)
        self.grl_layer = GradientReverseLayer()
       
        self.classifier_layer_list_11 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son1_1 = nn.Sequential(*self.classifier_layer_list_11)
        self.classifier_layer_list_12 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son1_2 = nn.Sequential(*self.classifier_layer_list_12)
        
        self.classifier_layer_list_21 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son2_1 = nn.Sequential(*self.classifier_layer_list_21)
        self.classifier_layer_list_22 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son2_2 = nn.Sequential(*self.classifier_layer_list_22)
        
        self.softmax = nn.Softmax(dim=1)

    def forward(self, data_src, data_tgt = 0, label_src = 0, mark = 1):
        mmd_loss = 0
        if self.training == True:
            if mark == 1:
                data_src = self.sharedNet(data_src)
                data_tgt = self.sharedNet(data_tgt)

                data_tgt_son1 = self.sonnet1(data_tgt)
                data_tgt_son1 = self.avgpool(data_tgt_son1)
                data_tgt_son1 = data_tgt_son1.view(data_tgt_son1.size(0), -1)

                data_src = self.sonnet1(data_src)
                data_src = self.avgpool(data_src)
                data_src = data_src.view(data_src.size(0), -1)
                
                inputs = torch.cat((data_src, data_tgt_son1), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son1_2(features_adv)
        
                outputs = self.classifier_layer_son1_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src.size(0)), label_src)## 源域分类损失
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt
                
                ###  match conditional distribution for ######################
                data_tgt_son2 = self.sonnet2(data_tgt)
                data_tgt_son2 = self.avgpool(data_tgt_son2)
                data_tgt_son2 = data_tgt_son2.view(data_tgt_son2.size(0), -1)
                data_tgt_son2 = self.classifier_layer_son2_1(data_tgt_son2)
                data_tgt_son1 = outputs.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0))
                
                l1_loss = torch.abs(torch.nn.functional.softmax(data_tgt_son1, dim=1) - torch.nn.functional.softmax(data_tgt_son2, dim=1))
                l1_loss = torch.mean(l1_loss)
                #########################################

                return classifier_loss, transfer_loss, l1_loss

            if mark == 2:
                data_src = self.sharedNet(data_src)
                data_tgt = self.sharedNet(data_tgt)

                data_tgt_son2 = self.sonnet2(data_tgt)
                data_tgt_son2 = self.avgpool(data_tgt_son2)
                data_tgt_son2 = data_tgt_son2.view(data_tgt_son2.size(0), -1)

                data_src = self.sonnet2(data_src)
                data_src = self.avgpool(data_src)
                data_src = data_src.view(data_src.size(0), -1)
                
                inputs = torch.cat((data_src, data_tgt_son2), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son2_2(features_adv)
        
                outputs = self.classifier_layer_son2_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src.size(0)), label_src)
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt
                
                ###  match conditional distribution for ######################
                data_tgt_son1 = self.sonnet1(data_tgt)
                data_tgt_son1 = self.avgpool(data_tgt_son1)
                data_tgt_son1 = data_tgt_son1.view(data_tgt_son1.size(0), -1)
                data_tgt_son1 = self.classifier_layer_son1_1(data_tgt_son1)
                data_tgt_son2 = outputs.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0))
                
                l1_loss = torch.abs(torch.nn.functional.softmax(data_tgt_son1, dim=1) - torch.nn.functional.softmax(data_tgt_son2, dim=1))
                l1_loss = torch.mean(l1_loss)

                return classifier_loss, transfer_loss, l1_loss

        else:
            data = self.sharedNet(data_src)

            fea_son1 = self.sonnet1(data)
            fea_son1 = self.avgpool(fea_son1)
            fea_son1 = fea_son1.view(fea_son1.size(0), -1)
            pred1 = self.classifier_layer_son1_1(fea_son1)

            fea_son2 = self.sonnet2(data)
            fea_son2 = self.avgpool(fea_son2)
            fea_son2 = fea_son2.view(fea_son2.size(0), -1)
            pred2 = self.classifier_layer_son2_1(fea_son2)

            return pred1, pred2


class MYDNN3_COM_MOE(nn.Module):
    def __init__(self, num_classes=31):
        super(MYDNN3_MOE, self).__init__()
        self.sharedNet = resnet50(True)
        self.sonnet1 = ADDneck(2048, 256)
        self.sonnet2 = ADDneck(2048, 256)
        self.sonnet3 = ADDneck(2048, 256)
        self.avgpool = nn.AvgPool2d(7, stride=1)
        self.grl_layer = GradientReverseLayer()
        
        self.classifier_layer_list_11 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son1_1 = nn.Sequential(*self.classifier_layer_list_11)
        self.classifier_layer_list_12 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son1_2 = nn.Sequential(*self.classifier_layer_list_12)
        
        self.classifier_layer_list_21 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son2_1 = nn.Sequential(*self.classifier_layer_list_21)
        self.classifier_layer_list_22 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son2_2 = nn.Sequential(*self.classifier_layer_list_22)

        self.classifier_layer_list_31 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son3_1 = nn.Sequential(*self.classifier_layer_list_31)
        self.classifier_layer_list_32 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son3_2 = nn.Sequential(*self.classifier_layer_list_32)
        
        self.softmax = nn.Softmax(dim=1)

    def forward(self, data_src1, data_src2=0,data_src3=0,data_tgt = 0, label_src1 = 0, label_src2 = 0,label_src3 = 0, mark = 1):
        mmd_loss = 0
        if self.training == True:
            data_src1 = self.sharedNet(data_src1)
            data_src2 = self.sharedNet(data_src2)
            data_src3 = self.sharedNet(data_src3)
            data_tgt = self.sharedNet(data_tgt)

            data_src1 = self.sonnet1(data_src1)
            data_src1 = self.avgpool(data_src1)
            data_src1 = data_src1.view(data_src1.size(0), -1)

            ## add for moe loss ####
            data_src2 = self.sonnet2(data_src2)
            data_src2 = self.avgpool(data_src2)
            data_src2 = data_src2.view(data_src2.size(0), -1)

            data_src3 = self.sonnet3(data_src3)
            data_src3 = self.avgpool(data_src3)
            data_src3 = data_src3.view(data_src3.size(0), -1) 
            if mark == 1:
                data_tgt_son1 = self.sonnet1(data_tgt)
                data_tgt_son1 = self.avgpool(data_tgt_son1)
                data_tgt_son1 = data_tgt_son1.view(data_tgt_son1.size(0), -1)

                outpus_data_src1_2 = self.classifier_layer_son2_1(data_src1) 
                outpus_data_src1_3 = self.classifier_layer_son3_1(data_src1) 
                
                inputs = torch.cat((data_src1, data_tgt_son1), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son1_2(features_adv)
        
                outputs = self.classifier_layer_son1_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src1.size(0)), label_src1)
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src1.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src1.size(0), inputs.size(0) - label_src1.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src1.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src1.size(0), inputs.size(0) - label_src1.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt
                ################# l1_loss ################
                #l1_loss = torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son1, dim=1)
                #                                - torch.nn.functional.softmax(data_tgt_son2, dim=1)) )
                #l1_loss += torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son1, dim=1)
                #                                - torch.nn.functional.softmax(data_tgt_son3, dim=1)) )
                return classifier_loss, transfer_loss, data_src2, data_src3, [outpus_data_src1_2, outpus_data_src1_3]

            if mark == 2:
                data_tgt_son2 = self.sonnet2(data_tgt)
                data_tgt_son2 = self.avgpool(data_tgt_son2)
                data_tgt_son2 = data_tgt_son2.view(data_tgt_son2.size(0), -1)

                outpus_data_src2_1 = self.classifier_layer_son1_1(data_src2) 
                outpus_data_src2_3 = self.classifier_layer_son3_1(data_src2) 
                
                inputs = torch.cat((data_src2, data_tgt_son2), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son2_2(features_adv)
        
                outputs = self.classifier_layer_son2_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src2.size(0)), label_src2)
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src2.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src2.size(0), inputs.size(0) - label_src2.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src2.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src2.size(0), inputs.size(0) - label_src2.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt
                
                ##############   ######
                '''
                l1_loss = torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son2, dim=1)
                                                - torch.nn.functional.softmax(data_tgt_son1, dim=1)) )
                l1_loss += torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son2, dim=1)
                                                - torch.nn.functional.softmax(data_tgt_son3, dim=1)) )
                '''
                return classifier_loss, transfer_loss, data_src1, data_src3, [outpus_data_src2_1, outpus_data_src2_3]

            if mark == 3:
                data_tgt_son3 = self.sonnet3(data_tgt)
                data_tgt_son3 = self.avgpool(data_tgt_son3)
                data_tgt_son3 = data_tgt_son3.view(data_tgt_son3.size(0), -1)

                outpus_data_src3_1 = self.classifier_layer_son1_1(data_src3) 
                outpus_data_src3_2 = self.classifier_layer_son2_1(data_src3) 
                
                inputs = torch.cat((data_src3, data_tgt_son3), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son3_2(features_adv)
        
                outputs = self.classifier_layer_son3_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src3.size(0)), label_src3)
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src3.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src3.size(0), inputs.size(0) - label_src3.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src3.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src3.size(0), inputs.size(0) - label_src3.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt
                
                #######################################################
                '''
                l1_loss = torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son3, dim=1)
                                                - torch.nn.functional.softmax(data_tgt_son1, dim=1)) )
                l1_loss += torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son3, dim=1)
                                                - torch.nn.functional.softmax(data_tgt_son2, dim=1)) )
                '''
                return classifier_loss, transfer_loss, data_src1, data_src2, [outpus_data_src3_1, outpus_data_src3_2]

        else:
            data = self.sharedNet(data_src1)

            fea_son1 = self.sonnet1(data)
            fea_son1 = self.avgpool(fea_son1)
            fea_son1 = fea_son1.view(fea_son1.size(0), -1)
            pred1 = self.classifier_layer_son1_1(fea_son1)

            fea_son2 = self.sonnet2(data)
            fea_son2 = self.avgpool(fea_son2)
            fea_son2 = fea_son2.view(fea_son2.size(0), -1)
            pred2 = self.classifier_layer_son2_1(fea_son2)

            fea_son3 = self.sonnet3(data)
            fea_son3 = self.avgpool(fea_son3)
            fea_son3 = fea_son3.view(fea_son3.size(0), -1)
            pred3 = self.classifier_layer_son3_1(fea_son3)

            return [pred1, pred2, pred3],[fea_son1,fea_son2,fea_son3]


class MYDNN3_MOE(nn.Module):
    def __init__(self, num_classes=31):
        super(MYDNN3_MOE, self).__init__()
        self.sharedNet = resnet50(True)
        self.sonnet1 = ADDneck(2048, 256)
        self.sonnet2 = ADDneck(2048, 256)
        self.sonnet3 = ADDneck(2048, 256)
        self.avgpool = nn.AvgPool2d(7, stride=1)
        self.grl_layer = GradientReverseLayer()
        
        self.classifier_layer_list_11 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son1_1 = nn.Sequential(*self.classifier_layer_list_11)
        self.classifier_layer_list_12 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son1_2 = nn.Sequential(*self.classifier_layer_list_12)
        
        self.classifier_layer_list_21 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son2_1 = nn.Sequential(*self.classifier_layer_list_21)
        self.classifier_layer_list_22 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son2_2 = nn.Sequential(*self.classifier_layer_list_22)

        self.classifier_layer_list_31 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son3_1 = nn.Sequential(*self.classifier_layer_list_31)
        self.classifier_layer_list_32 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son3_2 = nn.Sequential(*self.classifier_layer_list_32)
        
        self.softmax = nn.Softmax(dim=1)

    def forward(self, data_src1, data_src2=0,data_src3=0,data_tgt = 0, label_src1 = 0, label_src2 = 0,label_src3 = 0, mark = 1):
        mmd_loss = 0
        if self.training == True:
            data_src1 = self.sharedNet(data_src1)
            data_src2 = self.sharedNet(data_src2)
            data_src3 = self.sharedNet(data_src3)
            data_tgt = self.sharedNet(data_tgt)

            data_src1 = self.sonnet1(data_src1)
            data_src1 = self.avgpool(data_src1)
            data_src1 = data_src1.view(data_src1.size(0), -1)

            ## add for moe loss ####
            data_src2 = self.sonnet2(data_src2)
            data_src2 = self.avgpool(data_src2)
            data_src2 = data_src2.view(data_src2.size(0), -1)

            data_src3 = self.sonnet3(data_src3)
            data_src3 = self.avgpool(data_src3)
            data_src3 = data_src3.view(data_src3.size(0), -1) 
            if mark == 1:
                data_tgt_son1 = self.sonnet1(data_tgt)
                data_tgt_son1 = self.avgpool(data_tgt_son1)
                data_tgt_son1 = data_tgt_son1.view(data_tgt_son1.size(0), -1)

                outpus_data_src1_2 = self.classifier_layer_son2_1(data_src1) 
                outpus_data_src1_3 = self.classifier_layer_son3_1(data_src1) 
                
                inputs = torch.cat((data_src1, data_tgt_son1), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son1_2(features_adv)
        
                outputs = self.classifier_layer_son1_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src1.size(0)), label_src1)
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src1.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src1.size(0), inputs.size(0) - label_src1.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src1.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src1.size(0), inputs.size(0) - label_src1.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt
                ################# l1_loss ################
                #l1_loss = torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son1, dim=1)
                #                                - torch.nn.functional.softmax(data_tgt_son2, dim=1)) )
                #l1_loss += torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son1, dim=1)
                #                                - torch.nn.functional.softmax(data_tgt_son3, dim=1)) )
                return classifier_loss, transfer_loss, data_src2, data_src3, [outpus_data_src1_2, outpus_data_src1_3]

            if mark == 2:
                data_tgt_son2 = self.sonnet2(data_tgt)
                data_tgt_son2 = self.avgpool(data_tgt_son2)
                data_tgt_son2 = data_tgt_son2.view(data_tgt_son2.size(0), -1)

                outpus_data_src2_1 = self.classifier_layer_son1_1(data_src2) 
                outpus_data_src2_3 = self.classifier_layer_son3_1(data_src2) 
                
                inputs = torch.cat((data_src2, data_tgt_son2), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son2_2(features_adv)
        
                outputs = self.classifier_layer_son2_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src2.size(0)), label_src2)
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src2.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src2.size(0), inputs.size(0) - label_src2.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src2.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src2.size(0), inputs.size(0) - label_src2.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt
                
                ##############   ######
                '''
                l1_loss = torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son2, dim=1)
                                                - torch.nn.functional.softmax(data_tgt_son1, dim=1)) )
                l1_loss += torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son2, dim=1)
                                                - torch.nn.functional.softmax(data_tgt_son3, dim=1)) )
                '''
                return classifier_loss, transfer_loss, data_src1, data_src3, [outpus_data_src2_1, outpus_data_src2_3]

            if mark == 3:
                data_tgt_son3 = self.sonnet3(data_tgt)
                data_tgt_son3 = self.avgpool(data_tgt_son3)
                data_tgt_son3 = data_tgt_son3.view(data_tgt_son3.size(0), -1)

                outpus_data_src3_1 = self.classifier_layer_son1_1(data_src3) 
                outpus_data_src3_2 = self.classifier_layer_son2_1(data_src3) 
                
                inputs = torch.cat((data_src3, data_tgt_son3), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son3_2(features_adv)
        
                outputs = self.classifier_layer_son3_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src3.size(0)), label_src3)
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src3.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src3.size(0), inputs.size(0) - label_src3.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src3.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src3.size(0), inputs.size(0) - label_src3.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt
                
                #######################################################
                '''
                l1_loss = torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son3, dim=1)
                                                - torch.nn.functional.softmax(data_tgt_son1, dim=1)) )
                l1_loss += torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son3, dim=1)
                                                - torch.nn.functional.softmax(data_tgt_son2, dim=1)) )
                '''
                return classifier_loss, transfer_loss, data_src1, data_src2, [outpus_data_src3_1, outpus_data_src3_2]

        else:
            data = self.sharedNet(data_src1)

            fea_son1 = self.sonnet1(data)
            fea_son1 = self.avgpool(fea_son1)
            fea_son1 = fea_son1.view(fea_son1.size(0), -1)
            pred1 = self.classifier_layer_son1_1(fea_son1)

            fea_son2 = self.sonnet2(data)
            fea_son2 = self.avgpool(fea_son2)
            fea_son2 = fea_son2.view(fea_son2.size(0), -1)
            pred2 = self.classifier_layer_son2_1(fea_son2)

            fea_son3 = self.sonnet3(data)
            fea_son3 = self.avgpool(fea_son3)
            fea_son3 = fea_son3.view(fea_son3.size(0), -1)
            pred3 = self.classifier_layer_son3_1(fea_son3)

            return [pred1, pred2, pred3],[fea_son1,fea_son2,fea_son3]

class MYDNN3(nn.Module):
    def __init__(self, num_classes=31):
        super(MYDNN3, self).__init__()
        self.sharedNet = resnet50(True)
        self.sonnet1 = ADDneck(2048, 256)
        self.sonnet2 = ADDneck(2048, 256)
        self.sonnet3 = ADDneck(2048, 256)
        self.avgpool = nn.AvgPool2d(7, stride=1)
        self.grl_layer = GradientReverseLayer()
        
        self.classifier_layer_list_11 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son1_1 = nn.Sequential(*self.classifier_layer_list_11)
        self.classifier_layer_list_12 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son1_2 = nn.Sequential(*self.classifier_layer_list_12)
        
        self.classifier_layer_list_21 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son2_1 = nn.Sequential(*self.classifier_layer_list_21)
        self.classifier_layer_list_22 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son2_2 = nn.Sequential(*self.classifier_layer_list_22)

        self.classifier_layer_list_31 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son3_1 = nn.Sequential(*self.classifier_layer_list_31)
        self.classifier_layer_list_32 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son3_2 = nn.Sequential(*self.classifier_layer_list_32)
        
        self.softmax = nn.Softmax(dim=1)

    def forward(self, data_src, data_tgt = 0, label_src = 0, mark = 1):
        mmd_loss = 0
        if self.training == True:
            if mark == 1:
                data_src = self.sharedNet(data_src)
                data_tgt = self.sharedNet(data_tgt)

                data_tgt_son1 = self.sonnet1(data_tgt)
                data_tgt_son1 = self.avgpool(data_tgt_son1)
                data_tgt_son1 = data_tgt_son1.view(data_tgt_son1.size(0), -1)

                data_src = self.sonnet1(data_src)
                data_src = self.avgpool(data_src)
                data_src = data_src.view(data_src.size(0), -1)
                
                inputs = torch.cat((data_src, data_tgt_son1), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son1_2(features_adv)
        
                outputs = self.classifier_layer_son1_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src.size(0)), label_src)
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt

                return classifier_loss, transfer_loss

            if mark == 2:
                data_src = self.sharedNet(data_src)
                data_tgt = self.sharedNet(data_tgt)

                data_tgt_son2 = self.sonnet2(data_tgt)
                data_tgt_son2 = self.avgpool(data_tgt_son2)
                data_tgt_son2 = data_tgt_son2.view(data_tgt_son2.size(0), -1)

                data_src = self.sonnet2(data_src)
                data_src = self.avgpool(data_src)
                data_src = data_src.view(data_src.size(0), -1)
                
                inputs = torch.cat((data_src, data_tgt_son2), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son2_2(features_adv)
        
                outputs = self.classifier_layer_son2_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src.size(0)), label_src)
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt

                return classifier_loss, transfer_loss

            if mark == 3:
                data_src = self.sharedNet(data_src)
                data_tgt = self.sharedNet(data_tgt)

                data_tgt_son3 = self.sonnet3(data_tgt)
                data_tgt_son3 = self.avgpool(data_tgt_son3)
                data_tgt_son3 = data_tgt_son3.view(data_tgt_son3.size(0), -1)

                data_src = self.sonnet3(data_src)
                data_src = self.avgpool(data_src)
                data_src = data_src.view(data_src.size(0), -1)
                
                inputs = torch.cat((data_src, data_tgt_son3), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son3_2(features_adv)
        
                outputs = self.classifier_layer_son3_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src.size(0)), label_src)
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt

                return classifier_loss, transfer_loss

        else:
            data = self.sharedNet(data_src)

            fea_son1 = self.sonnet1(data)
            fea_son1 = self.avgpool(fea_son1)
            fea_son1 = fea_son1.view(fea_son1.size(0), -1)
            pred1 = self.classifier_layer_son1_1(fea_son1)

            fea_son2 = self.sonnet2(data)
            fea_son2 = self.avgpool(fea_son2)
            fea_son2 = fea_son2.view(fea_son2.size(0), -1)
            pred2 = self.classifier_layer_son2_1(fea_son2)

            fea_son3 = self.sonnet3(data)
            fea_son3 = self.avgpool(fea_son3)
            fea_son3 = fea_son3.view(fea_son3.size(0), -1)
            pred3 = self.classifier_layer_son3_1(fea_son3)

            return pred1, pred2, pred3
        

class MYDNN3_Conditional(nn.Module):
    def __init__(self, num_classes=31):
        super(MYDNN3_Conditional, self).__init__()
        self.sharedNet = resnet50(True)
        self.sonnet1 = ADDneck(2048, 256)
        self.sonnet2 = ADDneck(2048, 256)
        self.sonnet3 = ADDneck(2048, 256)
        self.avgpool = nn.AvgPool2d(7, stride=1)
        self.grl_layer = GradientReverseLayer()
        
        self.classifier_layer_list_11 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son1_1 = nn.Sequential(*self.classifier_layer_list_11)
        self.classifier_layer_list_12 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son1_2 = nn.Sequential(*self.classifier_layer_list_12)
        
        self.classifier_layer_list_21 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son2_1 = nn.Sequential(*self.classifier_layer_list_21)
        self.classifier_layer_list_22 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son2_2 = nn.Sequential(*self.classifier_layer_list_22)

        self.classifier_layer_list_31 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son3_1 = nn.Sequential(*self.classifier_layer_list_31)
        self.classifier_layer_list_32 = [nn.Linear(256, num_classes)]
        self.classifier_layer_son3_2 = nn.Sequential(*self.classifier_layer_list_32)
        
        self.softmax = nn.Softmax(dim=1)

    def forward(self, data_src, data_tgt = 0, label_src = 0, mark = 1):
        mmd_loss = 0
        if self.training == True:
            data_src = self.sharedNet(data_src)
            data_tgt = self.sharedNet(data_tgt)

            data_tgt_son1 = self.sonnet1(data_tgt)
            data_tgt_son1 = self.avgpool(data_tgt_son1)
            data_tgt_son1 = data_tgt_son1.view(data_tgt_son1.size(0), -1)
            pred_tgt_son1 = self.classifier_layer_son1_1(data_tgt_son1)

            data_tgt_son2 = self.sonnet2(data_tgt)
            data_tgt_son2 = self.avgpool(data_tgt_son2)
            data_tgt_son2 = data_tgt_son2.view(data_tgt_son2.size(0), -1)
            pred_tgt_son2 = self.classifier_layer_son2_1(data_tgt_son2)

            data_tgt_son3 = self.sonnet3(data_tgt)
            data_tgt_son3 = self.avgpool(data_tgt_son3)
            data_tgt_son3 = data_tgt_son3.view(data_tgt_son3.size(0), -1)
            pred_tgt_son3 = self.classifier_layer_son3_1(data_tgt_son3)
            
            if mark == 1:
                data_tgt_son1 = self.sonnet1(data_tgt)
                data_tgt_son1 = self.avgpool(data_tgt_son1)
                data_tgt_son1 = data_tgt_son1.view(data_tgt_son1.size(0), -1)

                data_src = self.sonnet1(data_src)
                data_src = self.avgpool(data_src)
                data_src = data_src.view(data_src.size(0), -1)
                
                inputs = torch.cat((data_src, data_tgt_son1), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son1_2(features_adv)
        
                outputs = self.classifier_layer_son1_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src.size(0)), label_src)
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt
                ################# l1_loss ################
                l1_loss = torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son1, dim=1)
                                                - torch.nn.functional.softmax(data_tgt_son2, dim=1)) )
                l1_loss += torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son1, dim=1)
                                                - torch.nn.functional.softmax(data_tgt_son3, dim=1)) )

                return classifier_loss, transfer_loss, l1_loss

            if mark == 2:

                data_tgt_son2 = self.sonnet2(data_tgt)
                data_tgt_son2 = self.avgpool(data_tgt_son2)
                data_tgt_son2 = data_tgt_son2.view(data_tgt_son2.size(0), -1)

                data_src = self.sonnet2(data_src)
                data_src = self.avgpool(data_src)
                data_src = data_src.view(data_src.size(0), -1)
                
                inputs = torch.cat((data_src, data_tgt_son2), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son2_2(features_adv)
        
                outputs = self.classifier_layer_son2_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src.size(0)), label_src)
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt
                
                ##############   ######
                l1_loss = torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son2, dim=1)
                                                - torch.nn.functional.softmax(data_tgt_son1, dim=1)) )
                l1_loss += torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son2, dim=1)
                                                - torch.nn.functional.softmax(data_tgt_son3, dim=1)) )

                return classifier_loss, transfer_loss, l1_loss

            if mark == 3:

                data_tgt_son3 = self.sonnet3(data_tgt)
                data_tgt_son3 = self.avgpool(data_tgt_son3)
                data_tgt_son3 = data_tgt_son3.view(data_tgt_son3.size(0), -1)

                data_src = self.sonnet3(data_src)
                data_src = self.avgpool(data_src)
                data_src = data_src.view(data_src.size(0), -1)
                
                inputs = torch.cat((data_src, data_tgt_son3), dim=0)
                features_adv = self.grl_layer(inputs)
                outputs_adv = self.classifier_layer_son3_2(features_adv)
        
                outputs = self.classifier_layer_son3_1(inputs)
                softmax_outputs = self.softmax(outputs)
            
                class_criterion = nn.CrossEntropyLoss()
                classifier_loss = class_criterion(outputs.narrow(0, 0, label_src.size(0)), label_src)
            
                target_adv = outputs.max(1)[1] ##返回的是 每行最大值所在的下标，也就是分类的结果
                target_adv_src = target_adv.narrow(0, 0, label_src.size(0)) ## 源域的分类结果
                target_adv_tgt = target_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0))## 目标域的分类结果

                #源域数据经过两个分类器后的分类结果要一致
                classifier_loss_adv_src = class_criterion(outputs_adv.narrow(0, 0, label_src.size(0)), target_adv_src)

                ## 目标域数据经过两个分类器后，分类结果要一致，论文中对目标域的处理跟源域的不一样
                logloss_tgt = torch.log(1 - F.softmax(outputs_adv.narrow(0, label_src.size(0), inputs.size(0) - label_src.size(0)), dim = 1))
                classifier_loss_adv_tgt = F.nll_loss(logloss_tgt, target_adv_tgt)

                transfer_loss = 3 * classifier_loss_adv_src + classifier_loss_adv_tgt
                
                #######################################################
                l1_loss = torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son3, dim=1)
                                                - torch.nn.functional.softmax(data_tgt_son1, dim=1)) )
                l1_loss += torch.mean( torch.abs(torch.nn.functional.softmax(data_tgt_son3, dim=1)
                                                - torch.nn.functional.softmax(data_tgt_son2, dim=1)) )

                return classifier_loss, transfer_loss, l1_loss

        else:
            data = self.sharedNet(data_src)

            fea_son1 = self.sonnet1(data)
            fea_son1 = self.avgpool(fea_son1)
            fea_son1 = fea_son1.view(fea_son1.size(0), -1)
            pred1 = self.classifier_layer_son1_1(fea_son1)

            fea_son2 = self.sonnet2(data)
            fea_son2 = self.avgpool(fea_son2)
            fea_son2 = fea_son2.view(fea_son2.size(0), -1)
            pred2 = self.classifier_layer_son2_1(fea_son2)

            fea_son3 = self.sonnet3(data)
            fea_son3 = self.avgpool(fea_son3)
            fea_son3 = fea_son3.view(fea_son3.size(0), -1)
            pred3 = self.classifier_layer_son3_1(fea_son3)

            return pred1, pred2, pred3
        

class MFSAN(nn.Module):

    def __init__(self, num_classes=31):
        super(MFSAN, self).__init__()
        self.sharedNet = resnet50(True)
        self.sonnet1 = ADDneck(2048, 256)
        self.sonnet2 = ADDneck(2048, 256)
        self.cls_fc_son1 = nn.Linear(256, num_classes)
        self.cls_fc_son2 = nn.Linear(256, num_classes)
        self.avgpool = nn.AvgPool2d(7, stride=1)

    def forward(self, data_src, data_tgt = 0, label_src = 0, mark = 1):
        mmd_loss = 0
        if self.training == True:
            if mark == 1:
                data_src = self.sharedNet(data_src)
                data_tgt = self.sharedNet(data_tgt)

                data_tgt_son1 = self.sonnet1(data_tgt)
                data_tgt_son1 = self.avgpool(data_tgt_son1)
                data_tgt_son1 = data_tgt_son1.view(data_tgt_son1.size(0), -1)

                data_src = self.sonnet1(data_src)
                data_src = self.avgpool(data_src)
                data_src = data_src.view(data_src.size(0), -1)
                mmd_loss += mmd.mmd(data_src, data_tgt_son1)
                #mmd_loss += mmd.CORAL(data_src, data_tgt_son1)

                data_tgt_son1 = self.cls_fc_son1(data_tgt_son1)

                data_tgt_son2 = self.sonnet2(data_tgt)
                data_tgt_son2 = self.avgpool(data_tgt_son2)
                data_tgt_son2 = data_tgt_son2.view(data_tgt_son2.size(0), -1)
                data_tgt_son2 = self.cls_fc_son2(data_tgt_son2)
                
                l1_loss = torch.abs(torch.nn.functional.softmax(data_tgt_son1, dim=1) - torch.nn.functional.softmax(data_tgt_son2, dim=1))
                l1_loss = torch.mean(l1_loss)
                pred_src = self.cls_fc_son1(data_src)

                cls_loss = F.nll_loss(F.log_softmax(pred_src, dim=1), label_src)

                return cls_loss, mmd_loss, l1_loss

            if mark == 2:
                data_src = self.sharedNet(data_src)
                data_tgt = self.sharedNet(data_tgt)

                data_tgt_son2 = self.sonnet2(data_tgt)
                data_tgt_son2 = self.avgpool(data_tgt_son2)
                data_tgt_son2 = data_tgt_son2.view(data_tgt_son2.size(0), -1)

                data_src = self.sonnet2(data_src)
                data_src = self.avgpool(data_src)
                data_src = data_src.view(data_src.size(0), -1)
                mmd_loss += mmd.mmd(data_src, data_tgt_son2)
                #mmd_loss += mmd.CORAL(data_src, data_tgt_son2)

                data_tgt_son2 = self.cls_fc_son2(data_tgt_son2)

                data_tgt_son1 = self.sonnet1(data_tgt)
                data_tgt_son1 = self.avgpool(data_tgt_son1)
                data_tgt_son1 = data_tgt_son1.view(data_tgt_son1.size(0), -1)
                data_tgt_son1 = self.cls_fc_son1(data_tgt_son1)
                l1_loss = torch.abs(torch.nn.functional.softmax(data_tgt_son1, dim=1) - torch.nn.functional.softmax(data_tgt_son2, dim=1))
                l1_loss = torch.mean(l1_loss)

                #l1_loss = F.l1_loss(torch.nn.functional.softmax(data_tgt_son1, dim=1), torch.nn.functional.softmax(data_tgt_son2, dim=1))

                pred_src = self.cls_fc_son2(data_src)
                cls_loss = F.nll_loss(F.log_softmax(pred_src, dim=1), label_src)

                return cls_loss, mmd_loss, l1_loss

        else:
            data = self.sharedNet(data_src)

            fea_son1 = self.sonnet1(data)
            fea_son1 = self.avgpool(fea_son1)
            fea_son1 = fea_son1.view(fea_son1.size(0), -1)
            pred1 = self.cls_fc_son1(fea_son1)

            fea_son2 = self.sonnet2(data)
            fea_son2 = self.avgpool(fea_son2)
            fea_son2 = fea_son2.view(fea_son2.size(0), -1)
            pred2 = self.cls_fc_son2(fea_son2)

            return pred1, pred2


def alexnet(pretrained=False, **kwargs):
    r"""AlexNet model architecture from the
    `"One weird trick..." <https://arxiv.org/abs/1404.5997>`_ paper.
    Args:
        pretrained (bool): If True, returns a model pre-trained on ImageNet
    """
    model = AlexNet(**kwargs)
    if pretrained:
        model_path = './alexnet.pth.tar'
        pretrained_model = torch.load(model_path)
        model.load_state_dict(pretrained_model['state_dict'])
    return model

